Spaceship = Class{
	init = function(self, mass, controllable, x, y, vx, vy, r, g, b)
		self.mass = mass
		if controllable then self.angle = math.angle(0, 0, vx, vy) end
		self.position = {
			x = x,
			y = y,
		}
		self.lastpos = {
			x = x,
			y = y,
		}
		self.velocity = {
			x = vx,
			y = vy,
		}
		self.color = {
			r = r,
			g = g,
			b = b,
		}
	end
}

function Spaceship:calcPosition()
	self.position.x = self.position.x + self.velocity.x*steptime
	self.position.y = self.position.y + self.velocity.y*steptime
end

function Spaceship:calcGravity()
	for _, entity in pairs(entities) do
		if entity ~= self then
			local dist = math.dist(self.position.x, self.position.y, entity.position.x, entity.position.y)
			local ang = math.angle(self.position.x, self.position.y, entity.position.x, entity.position.y)
			local strength = math.clamp(0, (entity.mass/dist^2)*steptime, 1)
			local dvx = math.cos(ang)*strength
			local dvy = math.sin(ang)*strength
			self.velocity.x = self.velocity.x + dvx
			self.velocity.y = self.velocity.y + dvy
		end
	end
	
	if not self.angle then return end
	if love.keyboard.isDown("left") then
		self.angle = self.angle - math.pi*steptime/360
	elseif love.keyboard.isDown("right") then
		self.angle = self.angle + math.pi*steptime/360
	end
	if love.keyboard.isDown("up") then
		local dvx = math.cos(self.angle)*steptime*shipacceleration
		local dvy = math.sin(self.angle)*steptime*shipacceleration
		self.velocity.x = self.velocity.x + dvx
		self.velocity.y = self.velocity.y + dvy
	end
end

-- auskommentieren, wenn du mit x Planeten schießen können willst.

function Spaceship:shoot()
	if not self.angle then return end
	local _, _, _, _, dist = garm.arrow(self.angle, 32, 2/3)
	local x, y = garm.kart(self.angle, dist/2)
	local vx, vy = garm.kart(self.angle, 0.3)
	table.insert(entities, Planet(1, false, self.position.x+x, self.position.y+y, vx, vy, 0, 0, 0))
end

function Spaceship:draw()
	-- render to canvas
	if trails then
		canvas:renderTo(function()
			love.graphics.setLineWidth(1)
			love.graphics.setColor(self.color.r, self.color.g, self.color.b)
			love.graphics.push()
			love.graphics.origin()
			love.graphics.translate(canvas:getWidth()/2, canvas:getHeight()/2)
			love.graphics.line(self.lastpos.x, self.lastpos.y, self.position.x, self.position.y)
			love.graphics.pop()
		end)
	end
	self.lastpos.x, self.lastpos.y = self.position.x, self.position.y -- outside the above loop etc. to avoid artifacts when toggling trails
	
	-- render spaceship
	love.graphics.setLineWidth(1)
	if not self.angle then -- spaceship points forward
		local ang = math.angle(0, 0, self.velocity.x, self.velocity.y) -- get the angle from the velocity vector
		local x1, y1, x2, y2, dist = garm.arrow(ang, 32, 2/3)
		local x, y = garm.kart(ang, dist/2)
		x, y = x + self.position.x, y + self.position.y
		love.graphics.setColor(self.color.r, self.color.g, self.color.b, 127)
		love.graphics.polygon("fill", x1+x, y1+y, x, y, x2+x, y2+y)
		love.graphics.setColor(0, 0, 0, 255)
		love.graphics.polygon("line", x1+x, y1+y, x, y, x2+x, y2+y)
	else
		local x1, y1, x2, y2, dist = garm.arrow(self.angle, 32, 2/3)
		local x, y = garm.kart(self.angle, dist/2)
		x, y = x + self.position.x, y + self.position.y
		love.graphics.setColor(self.color.r, self.color.g, self.color.b, 127)
		love.graphics.polygon("fill", x1+x, y1+y, x, y, x2+x, y2+y)
		love.graphics.setColor(0, 0, 0, 255)
		love.graphics.polygon("line", x1+x, y1+y, x, y, x2+x, y2+y)
		if love.keyboard.isDown("up") then -- accelerating
			local nx, ny = garm.kart(self.angle + math.pi, dist + 15)
			love.graphics.setColor(255, 255, 0, 127)
			love.graphics.polygon("fill", x+x1, y+y1, x+nx, y+ny, x+x2, y+y2)
			local nx, ny = garm.kart(self.angle + math.pi, dist + 10)
			love.graphics.setColor(255, 127, 0, 127)
			love.graphics.polygon("fill", x+x1, y+y1, x+nx, y+ny, x+x2, y+y2)
			local nx, ny = garm.kart(self.angle + math.pi, dist + 5)
			love.graphics.setColor(255, 0, 0, 127)
			love.graphics.polygon("fill", x+x1, y+y1, x+nx, y+ny, x+x2, y+y2)
		end
	end
end